import pandas as pd
from ... import SystemClass
from ..._formatters import __check_elements


def __build_pd_dicts(
    distSys: SystemClass, element_name: str, element_type: str
) -> dict:

    distSys.dss.PDElements.Name(str(element_type) + "." + str(element_name))
    typ = distSys.dss.CktElement.Name().replace("." + str(element_name), "")

    losses = distSys.dss.CktElement.Losses()

    return {
        "type": typ,
        "name": element_name,
        "kw_losses": losses[0] / 1000,
        "kvar_losses": losses[1] / 1000,
    }


def pd_element_loss(
    distSys: SystemClass, element_name: str, element_type: str
) -> pd.DataFrame:

    __check_elements(
        list((str(element_type) + "." + str(element_name)).split()),
        distSys.dss.Circuit.AllElementNames(),
    )
    pd_loss = []
    pd_loss.append(__build_pd_dicts(distSys, element_name, element_type))
    return pd.DataFrame(pd_loss)


def pd_element_loss_list(
    distSys: SystemClass, element_names: list, element_type: str
) -> pd.DataFrame:

    if element_type == "List":
        __check_elements(element_names, distSys.dss.Lines.AllNames())
    elif element_type == "Transformer":
        __check_elements(element_names, distSys.dss.Transformers.AllNames())

    return pd.DataFrame(
        tuple(
            map(
                lambda element_name: __build_pd_dicts(
                    distSys, element_name, element_type
                ),
                element_names,
            )
        )
    )


def get_all_line_losses(distSys: SystemClass) -> pd.DataFrame:

    element_type = "Line"

    return pd.DataFrame(
        tuple(
            map(
                lambda element_name: __build_pd_dicts(
                    distSys, element_name, element_type
                ),
                distSys.dss.Lines.AllNames(),
            )
        )
    )


def get_all_transformers_losses(distSys: SystemClass) -> pd.DataFrame:

    element_type = "Transformer"

    return pd.DataFrame(
        tuple(
            map(
                lambda element_name: __build_pd_dicts(
                    distSys, element_name, element_type
                ),
                distSys.dss.Transformers.AllNames(),
            )
        )
    )


def get_all_pd_elements_losses(distSys: SystemClass) -> pd.DataFrame:

    line_losses = get_all_line_losses(distSys)

    transformer_losses = get_all_transformers_losses(distSys)

    return pd.concat([transformer_losses, line_losses])


def get_total_pd_elements_losses(distSys: SystemClass) -> pd.DataFrame:
    data_loss = []
    data = {
        "name": "all_pd_elements",
        "kw_losses_total": sum(
            get_all_pd_elements_losses(distSys)["kw_losses"]
        ),
        "kvar_losses_total": sum(
            get_all_pd_elements_losses(distSys)["kw_losses"]
        ),
    }

    data_loss.append(data)
    return pd.DataFrame(data_loss)



import pandas
from functools import reduce
from .. import SystemClass
from ..decorators import actions, mode
from typing import Iterable, Tuple, Callable


@mode
def run_static_pf(
    distSys: SystemClass,
    actions: Iterable[Callable] = (lambda distSys: None,),
    tools: Iterable[Callable] = (lambda distSys: None,),
) -> tuple:
    """
    Run the static power flow mode.
    To see how it works, see [Learn DSSData](/tutorial/#static-power-flow).

    Args:
        distSys: An instance of [SystemClass][dssdata.SystemClass]
        actions: Actions functions.
        tools: Tools functions.

    Returns:
        Tools functions returns
    """
    [action(distSys) for action in actions]
    distSys.run_command("set mode=Snap")
    distSys.dss.Solution.Solve()
    return tuple(tool(distSys) for tool in tools)


@actions
def cfg_tspf(
    distSys: SystemClass, step_size: str = "1h", initial_time: tuple = (0, 0)
) -> None:
    """
    Set the time series mode in the distribution system.

    Args:
        distSys: An instance of [SystemClass][dssdata.SystemClass]
        step_size: The size of step time. See "Stepsize" in [OpenDSS User Manual](http://svn.code.sf.net/p/electricdss/code/trunk/Distrib/Doc/OpenDSSManual.pdf).
        initial_time:  See "Time" in [OpenDSS User Manual](http://svn.code.sf.net/p/electricdss/code/trunk/Distrib/Doc/OpenDSSManual.pdf).
    """  # noqa

    cmd = f"set mode=daily stepsize={step_size} "
    cmd2 = f'time = "{initial_time[0]}, {initial_time[1]}"'
    distSys.run_command(cmd + cmd2)


@mode
def run_tspf(
    distSys: SystemClass,
    num_steps: int,
    actions: Iterable[Callable] = (lambda distSys: None,),
    tools: Iterable[Callable] = (lambda distSys: None,),
) -> Tuple[pandas.DataFrame]:
    """
    Run the time series power flow.
    To see how it works, see [Learn DSSData](/tutorial/#time-series-power-flow).

    Args:
        distSys: An instance of [SystemClass][dssdata.SystemClass]
        num_steps : Number of time steps.
        actions: Actions functions.
        tools: Tools functions.
        

    Returns:
        Tools functions returns for all steps
    """  # noqa

    def concat_dfs(list_df1, list_df2):
        """
        Concatena dois conjuntos de DF par a par. Ex:
        list_df1 = [df1, df2, df3]
        list_df2 = [df4, df5, df6]
        return [pd.concat(df1,df4), pd.concat(df2,df5), pd.concat(df3,df6)]

        Args:
            list_df1 ([type]): [description]
            list_df2 ([type]): [description]

        Returns:
            [type]: [description]
        """
        return map(
            lambda df1, df2: pandas.concat([df1, df2], ignore_index=True),
            list_df1,
            list_df2,
        )

    def runmode_one_step(distSys):
        distSys.dss.Solution.Number(1)
        distSys.dss.Solution.Solve()

    def run_tool(distSys, tool, step):
        df = tool(distSys)
        df["step"] = step
        return df

    def run_concetps(distSys, step):
        [action(distSys) for action in actions]
        runmode_one_step(distSys)
        return tuple(run_tool(distSys, tool, step) for tool in tools)

    all_steps = (run_concetps(distSys, step) for step in range(0, num_steps))

    return tuple(reduce(concat_dfs, all_steps))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['dssdata',
 'dssdata.decorators',
 'dssdata.pfmodes',
 'dssdata.reductions',
 'dssdata.reductions.regs',
 'dssdata.tools',
 'dssdata.tools.lines',
 'dssdata.tools.losses',
 'dssdata.tools.regs',
 'dssdata.tools.voltages']

package_data = \
{'': ['*']}

install_requires = \
['OpenDSSDirect.py[extras]>=0.3.7,<0.4.0']

setup_kwargs = {
    'name': 'dssdata',
    'version': '0.1.2',
    'description': 'Organizing OpenDSS data',
    'long_description': '# DSSData\n\n\nA python micro-framework for simulation and data analysis of electrical distribution systems modeled on [OpenDSS](https://www.epri.com/#/pages/sa/opendss?lang=en).\n\nMode support: Static and Time-series\n\n\n## Documentation\n\nSee [DSSData Documentation](https://felipemarkson.github.io/power-flow-analysis/).\n\n## Installation\n\nWe strongly recommend the use of virtual environments manager.\n\n\n### Using pip\n\n```console\npip install dssdata\n```\n\n### Using poetry\n\n```console\npoetry add dssdata\n```\n\nContributors: \n\n- [JonasVil](https://github.com/felipemarkson/power-flow-analysis/commits?author=JonasVil)\n\n<!--\n### ```powerflow.losses_tools.get_total_pd_elements_losses(powerflow.systemclass.SystemClass)```\n\nRetorna um [pandas.Dataframe](https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.DataFrame.html) com os dados do somatório das perdas de todos os elementos do tipo PD (Power Delivery). Apresenta as perdas ativas(kW) e reativas(kVAr). Ex:\n\n|         name       |  kw_losses_total  | kvar_losses_total |\n|--------------------|-------------------|-------------------|\n|   all_pd_elements  |       112.398     |      327.926      |\n\nObs: Apesar dos capacitores serem tratados como um elemento do tipo PD, eles não são considerados.\n\n\n### ```powerflow.losses_tools.get_transformer_losses(powerflow.systemclass.SystemClass)```\nRetorna um [pandas.Dataframe](https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.DataFrame.html) com os dados referentes as perdas um a um dos transformadores conectaods à rede. Além disso, é apresentado o valor referente as perdas totais relacionadas aos transformadores. Apresenta as perdas ativas(kW) e reativas(kVAr).\n\n### ```powerflow.losses_tools.get_all_pd_elements_losses(powerflow.systemclass.SystemClass)```\nRetorna um [pandas.Dataframe](https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.DataFrame.html) com os dados referentes as perdas um a um dos elementos do tipo PD conectaods à rede. Apresenta as perdas ativas(kW) e reativas(kVAr).\n\nObs: Apesar dos capacitores serem tratados como um elemento do tipo PD, eles não são considerados.\n\n\n## Como contribuir\n\nEsteja livre para criar outras classes além do PowerFlow.\n\nTodos atributos das classes devem ser privados, ou seja, iniciar com __ (dois underlines). Com exceção dos atributos que dão acesso ao [OpenDSSDirect.py](https://github.com/dss-extensions/OpenDSSDirect.py)\n\nO acesso ou mudança de um atributo deve ser feita por um método público.\n\nDeve-se definir quais métodos devem ser públicos e quais devem ser privados.\n\nOs métodos privados devem iniciar com __ (dois underlines).\n\nDê preferência por criar funções (métodos) pequenas que possuem apenas uma única responsabilidade.\n\nOs nomes dos atributos e dos métodos devem ser claros e legíveis, não precisa economizar no tamanho do nome ;).\n\nEnvie commits pequenos com poucas alterações por vez.\n\n\n## Requisitos para Desenvolvimento\n\n[Poetry](https://python-poetry.org/)\n\n-->\n',
    'author': 'Felipe M. S. Monteiro',
    'author_email': 'fmarkson@outlook.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/felipemarkson/dssdata',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<3.8',
}


setup(**setup_kwargs)
